// ObjectData.cpp: implementation of the CObjectData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ObjectViewer.h"
#include "ObjectData.h"
#include "math.h"

#define PI 3.14159265358979323846

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CObjectData::CObjectData()
{
	fileType=FILETYPE;	//Tipo de arquivo 2d vlido
	lines=NULL;
	points=NULL;
	numLines=0;
	numPoints=0;
	objectName="";
		
	isCreated=false;
}

CObjectData::~CObjectData()
{
	free(points);
	free(lines);
}

void CObjectData::Serializa(CArchive & ar)
{
	CObject::Serialize(ar);
	int index;

	if (ar.IsStoring()) {
		ar << fileType;
		ar << objectName;
		ar << numPoints;
		ar << numLines;
		for (index=0;index<numPoints;index++){
			ar << points[index].x;
			ar << points[index].y;
		}
		for (index=0;index<numLines;index++){
			ar << lines[index].start;
			ar << lines[index].end;
		}
	} else {
		ar >> fileType;
		ar >> objectName;
		ar >> numPoints;
		ar >> numLines;
		
		if (isCreated){
			free(points);
			free(lines);
		}

		//Aloca o espao necessrio para os pontos
		points=(point2d_ptr)malloc(sizeof(point2d)*numPoints);

		for (index=0;index<numPoints;index++){
			ar >> points[index].x;
			ar >> points[index].y;
		}
		
		//Aloca o espao necessrio para as linhas
		lines=(line2d_ptr)malloc(sizeof(line2d)*numLines);

		for (index=0;index<numLines;index++){
			ar >> lines[index].start;
			ar >> lines[index].end;
		}

		isCreated=true;
	}
}

IMPLEMENT_SERIAL(CObjectData, CObject, 0)

void CObjectData::SetObjectName(CString name)
{
	objectName=name;
}

void CObjectData::GetObjectName(CString* name)
{
	*name=objectName;
}

void CObjectData::SetNumPoints(int nPoints)
{
	numPoints=nPoints;
}

int CObjectData::GetNumPoints()
{
	return numPoints;
}

void CObjectData::SetNumLines(int nLines)
{
	numLines=nLines;
}

int CObjectData::GetNumLines()
{
	return numLines;
}

void CObjectData::DrawObject(CDC* pDC, CRect retang)
{
	int index;
	int centerX=retang.Width()/2;
	int centerY=retang.Height()/2;
	CPen LightGrayPen;

	pDC->TextOut(0,0,"Object: "+objectName);
	
	//Cria a caneta LightGrayPen
	LightGrayPen.CreatePen(PS_SOLID, 0, RGB(205,205,205));
	
	//Desenha os eixos
	pDC->SelectObject(LightGrayPen);
	pDC->MoveTo(centerX,0);
	pDC->LineTo(centerX,retang.Height());
	pDC->MoveTo(0,centerY);
	pDC->LineTo(retang.Width(),centerY);

	pDC->SelectStockObject(BLACK_PEN);

	for (index=0;index<numLines;index++) {
		pDC->MoveTo((int)(points[lines[index].start].x+centerX),(int)(centerY-points[lines[index].start].y));
		pDC->LineTo((int)(points[lines[index].end].x+centerX),(int)(centerY-points[lines[index].end].y));
	}
}

void CObjectData::Rotate(float degrees, float ox, float oy)
{
	float radius=(float)((degrees*PI)/180);
	float nx, ny;
	int index;

	for (index=0;index<numPoints;index++) {
		points[index].x-=ox;
		points[index].y-=oy;

		nx=points[index].x*(float)cos(radius)-points[index].y*(float)sin(radius);
		ny=points[index].x*(float)sin(radius)+points[index].y*(float)cos(radius);

		points[index].x=nx+ox;
		points[index].y=ny+oy;
	}
}

void CObjectData::Translate(float dx, float dy)
{
	int index;
	
	for (index=0;index<numPoints;index++) {
		points[index].x+=dx;
		points[index].y+=dy;
	}
}

void CObjectData::Scale(float sx, float sy)
{
	int index;

	for (index=0;index<numPoints;index++) {
		points[index].x=points[index].x*sx;
		points[index].y=points[index].y*sy;
	}
}

void CObjectData::Mirror(int type)
{
	int index;

	switch (type){
	case XONLY:	for (index=0;index<numPoints;index++) {
					points[index].y*=-1;
				}
				break;
	case YONLY:	for (index=0;index<numPoints;index++) {
					points[index].x*=-1;
				}
				break;
	}
}

void CObjectData::LiberaMemoria()
{
	free(points);
	free(lines);
}
